/*
 * @(#)HelloAglet.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package examples.hello;

import com.ibm.aglet.*;
import com.ibm.aglet.event.*;
import com.ibm.aglet.util.*;

import com.ibm.agletx.util.SimpleItinerary;

import java.lang.InterruptedException;
import java.io.Externalizable;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;
import java.net.*;
import java.awt.*;
import java.util.Enumeration;

/**
 * <tt> HelloAglet </tt> is a mobile aglet that goes to a remote host
 * to say "Hello World" and then returns home and dies.
 * <p>1. Shows a dialog box to specify the destination server.
 * invoke handleMessage() methods.
 * <p>2. (Execution of "startTrip" message) Moves to the destination server
 * and send "sayHello" message.
 * to itself.
 * <p>3. (Execution of "sayHello" message) Display a string for 5 secs
 * and go back to the origin. Send "atHome" message to itself.
 * <p>4. (Execution of "atHome" message) Display a string "I'm back"
 * for 2 secs and destroy itself.
 *
 * @version     1.00    $Date: 98/07/06 12:05 $
 * @author      Danny B. Lange
 * @author      Mitsuru Oshima
 */
public class HelloAglet extends Aglet {

    /*
     * UI to interact with a User
     * this will be automatically disposed when the aglet is disposed
     */
    transient Frame my_dialog = new MyDialog(this);

    /*
     * message
     */
    String message = "Hello World!";

    /*
     * home address represented as a string
     */
    String home = null;

    /*
     * Itinerary
     */
    SimpleItinerary itinerary = null;

    /*
     * Initializes the aglet. 
     * Only called the very first time this aglet is created. 
     */
    public void onCreation(Object init) {
        itinerary = new SimpleItinerary(this);

        my_dialog.pack();
        my_dialog.resize(my_dialog.preferredSize());
        my_dialog.show();

        // Initialize the variables.
        home = getAgletContext().getHostingURL().toString();
    }

    /*
     * Handles the message
     * @param msg the message sent
     */
    public boolean handleMessage(Message msg) {
        if (msg.sameKind("atHome")) {
            atHome(msg);
        } else if (msg.sameKind("startTrip")) {
            startTrip(msg);
        } else if (msg.sameKind("sayHello")) {
            sayHello(msg);
        } else if (msg.sameKind("dialog")) {
            dialog(msg);
        } else {
            return false;
        }
        return true;
    }

    /*
     * Reports arrival home and disappears
     */
    public void atHome(Message msg) {
        // greetings
        setText("I'm back.");

        // keep showing the text
        waitMessage(2 * 1000);

        // dispose it self
        dispose();
    }

    /**
     * Strats the trip of this aglet to the destination.
     */
    public synchronized void startTrip(Message msg) {
        String destination = (String)msg.getArg();
        
        try {
            itinerary.go(destination, "sayHello");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /*
     * Say hello!
     */
    public void sayHello(Message msg) {
        // greetings
        setText(message);

        // keep showing the text
        waitMessage(5 * 1000);

        // back home
        try {
            setText("I'll go back to.. " + home);
            waitMessage(1000);
            itinerary.go(home, "atHome");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Creates and shows the dialog window. 
     * This aglet keeps the reference to the instance
     * of the Dialog to avoid opening multiple dialog windows.
     */
    public void dialog(Message msg) {
        // check and create a dialog box
        if (my_dialog == null) {
            my_dialog = new MyDialog(this);
            my_dialog.pack();
            my_dialog.resize(my_dialog.preferredSize());
        }
        
        // show the dialog box
        my_dialog.show();
    }
}


/*
 * MyDialog class is the window to be opened when the dialog required.
 * This is NOT a subclass of Dialog.
 */
class MyDialog extends Frame {

    /*
     * The aglet a user interacts with.
     */
    private HelloAglet aglet  = null;

    /*
     * UI Components
     */
    private AddressChooser dest = new AddressChooser();
    private TextField msg       = new TextField(18);
    private Button go           = new Button("GO!");
    private Button send         = new Button("Send CLONE!");
    private Button close        = new Button("CLOSE");

    /*
     * Constructs the dialog window
     * @param aglet The aglet the user interacts with.
     */
    MyDialog(HelloAglet aglet) {
        this.aglet = aglet;
        layoutComponents();
    }

    /*
     * Layouts all components
     */
    private void layoutComponents() {
        msg.setText(aglet.message);

        // Layouts components
        GridBagLayout grid = new GridBagLayout();
        GridBagConstraints cns = new GridBagConstraints();
        setLayout(grid);

        cns.weightx = 0.5;
        cns.ipadx = cns.ipady = 5;
        cns.fill = GridBagConstraints.HORIZONTAL;
        cns.insets = new Insets(5,5,5,5);

        cns.weightx = 1.0;
        cns.gridwidth = GridBagConstraints.REMAINDER;
        grid.setConstraints(dest, cns);
        add(dest);

        cns.gridwidth = GridBagConstraints.REMAINDER;
        cns.fill = GridBagConstraints.BOTH;
        cns.weightx = 1.0;
        cns.weighty = 1.0;
        cns.gridheight = 2;
        grid.setConstraints(msg, cns);
        add(msg);

        cns.weighty = 0.0;
        cns.fill = GridBagConstraints.NONE;
        cns.gridheight = 1;

        Panel p = new Panel();

        grid.setConstraints(p, cns);
        add(p);
        p.setLayout(new FlowLayout());
        p.add(go);
        p.add(send);
        p.add(close);
    }

    /**
     * Handles the events
     * @param ev the event to be handled
     */
    public boolean handleEvent(Event ev) {
        if (ev.id == Event.WINDOW_DESTROY) {
            hide();
            return true;
        }
        return super.handleEvent(ev);
    }

    /**
     * Handles the action
     * @param ev the event to be handled
     * @param arg the extra argument
     */
    public boolean action(Event ev, Object obj) {
        if (ev.target == send) {
            aglet.message = msg.getText();
	    try {
		AgletProxy p = (AgletProxy)aglet.clone();
		p.sendOnewayMessage(new Message("startTrip", dest.getAddress()));
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	} else if (ev.target == go) {
            aglet.message = msg.getText();
            aglet.handleMessage(new Message("startTrip", dest.getAddress()));
        } else if (ev.target == close) {
            hide();
        } else {
            return false;
        }
        return true;
    }
}
