/*
 * @(#)AgletInfo.java
 *
 * (c) Copyright IBM Corp. 1997, 1998
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package com.ibm.aglet;

import java.net.URL;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/**
 * AgletInfo class is a object which contains the information of the aglet.
 *
 * @version     1.10	$Date: 98/04/10 7:26p $
 * @author	Mitsuru Oshima
 */
final public class AgletInfo implements java.io.Serializable, Cloneable {

    static final long serialVersionUID = 5077220171230015552L;

    /**
     * Constructs a AgletInfo with the specified parameters.
     * This is used internally and the aglet programmers should not use
     * this API.
     *
     * @param aid     the aglet identifier.
     * @param address the address where the aglet currently resides.
     * @param classname the classname of the aglet.
     * @param codebase the codebase of the aglet classes.
     * @param origin   the address where the aglet was instantiated.
     * @param birthtime the time when the aglet was instantiated.
     * @param api_major_version major version.
     * @param api_minor_version minor version.
     * @param authorityName the authority's name of the aglet.
     * @param privilegeName the name of the privilege in the current
     *                      environment.
     */
    public AgletInfo(AgletID aid,
		     String address,
		     String classname,
		     URL codebase,
		     String origin,
		     long birthtime,
		     short api_major_version,
		     short api_minor_version,
		     String authorityName,
		     String privilegeName) {
	this.aid = aid;
	this.address   = address;
	this.classname = classname;
	this.codebase  = codebase.toExternalForm();
	this.origin    = origin;
	this.birthtime = birthtime;
	this.api_major_version = api_major_version;
	this.api_minor_version = api_minor_version;
	this.authorityName = authorityName;
	this.privilegeName = privilegeName;
    }

    /**
     * Gets the aglet id
     */
    public AgletID getAgletID() {
	return aid;
    }

    /**
     * Gets the class name of the aglet.
     */
    public String getAgletClassName() {
	return classname;
    }

    /**
     * Gets the origin address where the aglet was instantiated.
     */
    public String getOrigin() {
	return origin;
    }

    /**
     * Gets the codebase address where the aglet code resides.
     */
    public URL getCodeBase() {
	try {
	    return new URL(codebase);
	} catch (java.net.MalformedURLException ex) {
	    ex.printStackTrace();
	    return null;
	}
    }

    /**
     * Gets the address where the aglet currently resides.
     */
    public String getAddress() {
	return address;
    }

    /**
     * Gets the time when the aglet is created.
     */
    public long getCreationTime() {
	return birthtime;
    }

    /**
     * Gets the API major version to which the aglet is instantiated.
     */
    public short getAPIMajorVersion() {
	return api_major_version;
    }

    /**
     * Gets the API minor version to which the aglet is instantiated.
     */
    public short getAPIMinorVersion() {
	return api_minor_version;
    }

    /**
     * Gets the authority name. This will be replaced with new Security API.
     * Please do not use this method for the moment.
     */
    public String getAuthorityName() {
	return authorityName;
    }

    /**
     * Gets the privilege name.  This will be replaced with new Security API.
     * Please do not use this method for the moment.
     * @deprecated
     */
    public String getPrivilegeName() {
	return privilegeName;
    }

    /**
     * Gets a string representation of the object.
     */
    public String toString() {
	StringBuffer b = new StringBuffer();
	b.append("ID       : " + aid + '\n');
	b.append("Address  : " + address + '\n');
	b.append("ClassName: " + classname + '\n');
	b.append("Origin   : " + origin + '\n');
	b.append("CodeBase : " + codebase + '\n');
	return b.toString();
    }

    /**
     * @deprecated
     */
    public AgletInfo clone(String a, URL c) {
	try {
	    AgletInfo clone = (AgletInfo)clone();
	    clone.address = a;
	    clone.codebase = c.toExternalForm();
	    return clone;
	} catch (CloneNotSupportedException ex) {
	    return null;
	}
    }

    // static attributes
    private AgletID aid;
    private String classname;
    private String origin;
    private long birthtime;
    private short api_major_version;
    private short api_minor_version;

    //
    private String address = null;
    private String codebase = null;

    //
    private String authorityName = null;
    private transient String privilegeName = null;
}
