/*
 * @(#)AgletRuntime.java
 *
 * (c) Copyright IBM Corp. 1997, 1998
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package com.ibm.aglet.system;

import com.ibm.aglet.Message;
import com.ibm.aglet.AgletContext;
import com.ibm.aglet.Aglet;
import com.ibm.aglet.AgletID;
import com.ibm.aglet.AgletProxy;
import com.ibm.aglet.InvalidAgletException;
import java.io.IOException;
import java.net.URL;

/**
 * <tt>AgletRuntime</tt> class provides the way to access the information
 * of the local or remote context. Neither of an application nor aglet
 * can create its own instance of this runtime class.
 *
 * @version     1.10    $Date: 98/04/17 2:28p $
 * @author      Mitsuru Oshima
 * @see AgletRuntime#getAgletRuntime
 */
public abstract class AgletRuntime {

    final static String runtimePackagePrefix = "aglet.runtime.packagePrefix";

    /**
     * Create and initialize a runtime environment with a string array.
     * @param a string array which is typically given as a argument to
     *        <tt>main(String args[])</tt> function.
     */
    synchronized static public AgletRuntime init(String args[]) {
	if (runtime != null) {
	    throw new IllegalAccessError("Already Initialized");
	}

	String packagePrefix = "com.ibm.aglets";
	try {
	    packagePrefix = System.getProperty(runtimePackagePrefix,
					       "com.ibm.aglets");
	} catch (SecurityException e) {
	    // for Fiji
	}

	String classname = packagePrefix + ".AgletRuntime";

	try {
	    Class clazz = Class.forName( classname );
	    Object obj = clazz.newInstance();
	    if (obj instanceof AgletRuntime) {
		runtime = (AgletRuntime)obj;
	    } else {
		System.out.println("[ \""+ classname + "\" is not Runtime]");
	    }
	} catch (ClassNotFoundException ex) {
	    System.out.println("[ The class \""+ classname + "\" not found]");
	} catch (Exception ex) {
	    System.out.println("[ An instance of \""+ classname +
			       "\" cannot be created]");
	}

	runtime.initialize(args);
	return runtime;
    }

    /**
     * Gets the AgletRuntime object associated with the current
     * Java application.
     */
    synchronized static public AgletRuntime getAgletRuntime() {
	if (runtime == null) {
	    init(null);
	}
	return runtime;
    }

    /**
     * Initializes an AgletRuntime object with the given array of string.
     * This is typically an argument of <tt>main(String args[])</tt> function.
     * @param args arguments used to initialize
     * @exception IllegalAccessException if the instance has
     *            been already initialized.
     */
    abstract protected void initialize(String args[]);

    /**
     * Creates an DefaultAgletContext object given by the Framework
     * implementation with specified name. 
     * @exception SecurityException if the current execution is not allowed
     *  to create an AgletContext.
     */
    abstract public AgletContext createAgletContext(String name);

    /**
     * Returns the AgletContext which has the specified name.
     *
     * @param name the name of the context
     * @exception SecurityException if the current execution is not allowed
     *  to acccess the AgletContext.
     */
    abstract public AgletContext getAgletContext(String name);

    /**
     * Removes the specified aglet context from the runtime environment.
     * It is also removed from export list if it's exported.
     * @param cxt the context to be removed.
     */
    abstract public void removeAgletContext(AgletContext cxt);

    /**
     * Gets the contexts in the environment.
     */
    abstract public AgletContext[] getAgletContexts();

    /**
     * Gets an address of the server in URL.
     * @return an URL represents the address of the server.
     */
    abstract public URL getServerURL();

    /**
     * Shutdown all contexts in the runtime.
     */
    abstract public void shutdown();

    /**
     * Shutdown all contexts in the current runtime with the specific message
     * object. This messag object is delivered to all aglets in all contexts
     * before all aglets are killed.
     */
    abstract public void shutdown(Message msg);

    /**
     * Kill the specified aglet. This aglet have to be a local aglet
     * in this runtime.
     * @param proxy the aglet proxy object to kill.
     */
    abstract public void killAglet(AgletProxy proxy) throws InvalidAgletException;

    /**
     * [Preliminary]
     * Exports the specified aglet context. After this successfull
     * invocation, the specified context is visiable and accessible
     * from a remote host, which means that the aglet can be dispatched
     * into and also the remote operations like listing the aglet proxies
     * residing in the context are enabled.
     * @param cxt the context to export
     * @exception SecurityException if the current execution is not allowed
     *  to export an AgletContext.
    abstract public void exportAgletContext(AgletContext cxt);
     */

    /**
     * [Preliminary]
     * Exports the specified aglet. After this successfull
     * invocation, the specified aglet is visiable and accessible
     * from a remote host, which means that the aglet can be receive
     * remote messages from remote hosts.
     * @param aglet the aglet to export
     * @exception SecurityException if the current execution is not allowed
     *  to export an Aglet
    abstract public void exportAglet(Aglet aglet);
     */

    /*
    abstract public boolean isGuiAvailable();
    */
    private static AgletRuntime runtime = null;

    /* =============================================================
     *
     * CLIENT APIs
     *
     * ============================================================ */

    /**
     * Creates an aglet remotely within the specified context.
     *
     * @param contextAddress an address of context.
     * @param codebase an codebase for the aglet.
     * @param name name of aglets' class.
     * @param init an object passed as an initialize argument.
     * @see Aglet#onCreation
     * @see AgletContext#createAglet
     */
    abstract protected AgletProxy createAglet(String contextAddress,
					      URL codebase,
					      String name,
					      Object init) throws IOException;

    /**
     * Obtains the remote proxy for the aglet specified by the context and
     * id.
     * @param contextAddress specify context URL with a string.
     * @param id target aglet identifyer.
     */
    abstract protected AgletProxy getAgletProxy(String contextAddress, AgletID id) throws IOException;

    /**
     * Returns the proxies in the context specified by the contextAddress.
     * @param contextAddress specify context URL with a string.
     */
    abstract protected AgletProxy[] getAgletProxies(String contextAddress) throws IOException;

    /**
     * Returns an enumeration of proxies in the context specified by
     * the contextAddress.
     * @param contextAddress
    abstract public Enumeration getAgletProxies(String contextAddress, String authorityName);
     */
}
