/*
 * @(#)SeqItinerary.java
 *
 * (c) Copyright IBM Corp. 1996, 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package com.ibm.agletx.util;

import com.ibm.aglet.*;
import com.ibm.aglet.event.*;

import java.util.Vector;
import java.util.Enumeration;
import java.net.URL;

/**
 * Define an abstract interface for an aglet's itinerary.
 * An Itinerary is a set of pairs of the form [host, task] where the 
 * task should be performed upon arrival of the aglet to the host.
 * 
 * @version     1.10    $Date: 98/04/14 5:17p $
 * @author      Yariv Aridor
 * @see Task
 */
public abstract class SeqItinerary  extends MobilityAdapter implements java.io.Serializable {

    static final long serialVersionUID = 7979344708988677111L;

    /**
     * Constructor.
     * @param aglet the owner aglet
     */
    public SeqItinerary(Aglet aglet) {
	this.aglet = aglet;
	aglet.addMobilityListener(this);
    }


   /**
    *  Return the address of the origin
    */
    public String getOrigin () {
	return origin;
    }


   /**
    *  Return the task to be preformed at the current destination
    */
    protected Task getCurrentTask() {
	return getTaskAt(index);
    }
 
   /**
    *  Return the address of the current destination
    */
    public String getCurrentAddress () {
	return getAddressAt(index);
    }


   /**
    *  Empty the itineray
    */
    public void clear () {
	hosts = new Vector();
	tasks = new Vector();
	index = 0;
    }


    /**
     *  Return the size of the aglet's itinerary.
     */
    public int size() {
	return hosts.size();
    }


    /**
     *  Return an enumeration of all the addresses
     */
    public Enumeration addresses () {
	return hosts.elements();
    }


    /**
     *  Add a new destination
     *  @param the address of the host where the task is to be executed.
     */
    public void addAddress(String address) {
	addTask(address, (Task)null);
    }

    /**
     *  Add a new task
     *  @param task the task to be added
     *  @param the address of the host where the task is to be executed.
     */
    public void addTask(String address, Task task) {
	hosts.addElement(address);
	tasks.addElement(task);
    }

    /**
     *  Return the task at a specific index 
     */
    public Task getTaskAt(int index) {
	return (Task)tasks.elementAt(index);
    }


    /**
     *  Return the index of a specific task
     */
    public int indexOf(Task task) {
	return tasks.indexOf(task);
    }

    /**
     *  Remove a task at a specific index
     */
    public void removeTaskAt(int index) {
	tasks.removeElementAt(index);
	hosts.removeElementAt(index);
    }

    /**
     *  Return the index of a specific address
     */
    public int indexOf(String address) {
	return hosts.indexOf(address);
    } 

    /**
     * Return the address at the specified index.
     */
    public String getAddressAt(int index) {
	return (String) hosts.elementAt(index);
    }


    /**
     *  Handle exception during task execution
     *  @param ex the exception
     */
    public void handleException(Throwable ex) {
	ex.printStackTrace();
    }

   /**
    *  Handle exception during the travelling of the owner aglet
    *  (try to dispatch to the next destination).
    *  @param ex the exception
    */
    public void handleTripException(Throwable ex) {
	tryNext();
    }

    /**
     * Check if at the last destination
     */
    synchronized public boolean atLastDestination() {
	return index >= hosts.size();
    }
    

    protected void onTermination () {
    }


    /**
     * Go to the next address and perform the next task
     */
    public void goToNext() {
	try {
	    if (atLastDestination()) {
		onTermination();
		return;
	    }
	    URL address = new URL( (String) hosts.elementAt(index) );
	    aglet.dispatch( address );
	} catch (Throwable ex) {
	    if (ex instanceof ThreadDeath == false) {
		handleTripException(ex);
	    }
	}
    }

    /**
     * Start the trip defined in this itinerary
     */
    public void startTrip() {
	origin = aglet.getAgletContext().getHostingURL().toString();
	index = 0;
	goToNext();
    }

    private void tryNext() {
	// try next one
	index++;
	goToNext();
    }


    /**
     * This is not normally used by aglets programmers.
     */
    public void onArrival(MobilityEvent ev) {
	currentTarget = ev.getAgletProxy();
	try {
	    Task task = getCurrentTask();
	    if (task!=null) task.execute(this);
	} catch (Throwable ex) {
	    handleException(ex);
	} finally {
	    index ++;
	    if (atLastDestination() && repeat) {
		index = 0;
	    }
	}
	goToNext();
    }

    /**
     * Return the Proxy of the owner aglet
     */   
    public AgletProxy getOwnerAglet() {
	return aglet.getAgletContext().getAgletProxy(aglet.getAgletID());
    }


    /**
     * Define whether the itinerary is to be repeated (cyclic)
     */  
    public void setRepeat(boolean b) {
	repeat = b;
    }

    /**
     * Check if the itinerary is a cyclic one.
     */
    public boolean isRepeat() {
	return repeat;
    }

    void incIndex () {
	index ++;
    }

    protected Aglet aglet;
    private Vector tasks = new Vector();
    private Vector hosts = new Vector();
    private int index = 0; // should be improved to become private 
    transient protected AgletProxy currentTarget = null;
    private boolean repeat = false;    
    private String origin = null;
}
