/*
 * @(#)SeqPlanItinerary.java
 *
 * (c) Copyright IBM Corp. 1996, 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package com.ibm.agletx.util;

import com.ibm.aglet.*;
import com.ibm.aglet.event.*;

import java.util.Vector;
import java.util.Enumeration;
import java.net.URL;

/**
 * <tt> SeqPlanItinerary </tt> defines the trip plan which has a sequence of
 * place-message pair. For example, the plan defined like:
 * <pre>
 *   itinerary.addPlan("atp://first", "job1");
 *   itinerary.addPlan("atp://second", "job2");
 * </pre>
 * first dispatches an aglet to the <tt> "atp://first" </tt> and
 * sends <tt> new Message("job1") </tt> message to the aglet. After
 * that message handling is completed, the itinerary dispatches the aglet
 * to the next address, <tt> "atp://second" </tt>, and send the
 * corresponding message <tt> new Message("job2", init) </tt>
 * to the aglet again.
 * The order of plan is defined in the order <tt>addPlan</tt> method is called,
 * or you can insert a new plan item at the specified index by calling <tt>
 * insertPlanAt </tt>. To automatically dispatches an aglet to the next
 * address when the job is completed, the aglet have to have the following
 * block
 *
 * @version     1.10    $Date: 98/04/14 5:17p $
 * @author      Mitsuru Oshima
 * @author      Yariv Aridor
 */
public class SeqPlanItinerary extends SeqItinerary {

    static final long serialVersionUID = 7979344708988677116L;

    /**
     * Constructs a SeqPlanItinerary object with the specified owner aglet.
     * @param aglet the owner aglet
     */
    public SeqPlanItinerary(Aglet aglet) {
	super(aglet);
    }

    public Message getMessageAt(int index) {
	return ((SeqPlanTask)getTaskAt(index)).getMessage();
    }

    /**
     * Adds the new itinerary item of the form [address, message].
     * @param address  the address to go
     * @param msg      the message to be sent to the owner aglet
     */
    public void addPlan(String address, Message msg) {
	addTask(address, new SeqPlanTask(msg));
    }

    /**
     * Adds the new itinerary item of the form [address, message].
     * This is added at the end of plan.
     * @param address  the address to go
     * @param msg      the message to be sent to the owner aglet
     */
    public void addPlan(String address, String msg) {
	addPlan(address, new Message(msg));
    }

    public void removePlanAt(int index) {
	removeTaskAt(index);
    }

    public void onArrival(MobilityEvent ev) {
	try {
	    Task task = getCurrentTask();
	    incIndex(); // should be improved
	    task.execute(this);
	} catch (Exception ex) {
	    handleException(ex);
	}
    }
}

class SeqPlanTask extends Task implements Runnable  {
    
    private Message msg = null;
    private AgletProxy p = null;
    
    public SeqPlanTask (Message msg) {
	this.msg = msg;
    }

    public Message getMessage () {
	return msg;
    }

    private SeqItinerary itin = null;
    public void execute (SeqItinerary itin) throws Exception {
	this.itin=itin;
	new Thread(this).start();
    }

    public void run () {
	AgletProxy p = itin.getOwnerAglet();
	try { 
	    p.sendMessage(msg);
	} catch (Exception ex) {
	    itin.handleException(ex);
	}
	if (itin.atLastDestination() & itin.isRepeat()) { 
	    itin.startTrip();
	} 
	else { 
	    itin.goToNext();
	}
    }

}
