/*
 * @(#)SlaveItinerary.java
 *
 * (c) Copyright IBM Corp. 1996, 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */
package com.ibm.agletx.util;

import com.ibm.aglet.*;
import com.ibm.aglet.event.*;
import java.util.*;
import java.net.*;
import java.io.IOException;

/**
 * An Itinerary class to repeatedly perform a specific task in multiple 
 * destinations. 
 * <br>
 * Here is a typical usage of this class. 
 * <pre>
 *  private boolean lastTask = false;
 *  private SlaveItinerary itinerary  = null;
 *  class TaskA extends Task {
 *	public void execute(SeqItinerary itin) throws Exception {
 *	   // do some work
 *	}
 *  }
 *
 *  class TaskB extends Task {
 *	public void execute(SeqItinerary itin) throws Exception {
 *	    // do some work
 *	}
 *  }
 *
 *  public void onCreation(Object ini) {	
 *	    itinerary = new SlaveItinerary(this,"atp://yariv.trl.ibm.com",new TaskA());
 *            itinerary.addPlace("atp://tai.trl.ibm.com");
 *	    itinerary.startTrip();
 *   }
 *
 *  public void run () {
 *	if (itinerary.atLastDestination()==true) {
 *	    if (lastTask==true) { 
 *		//  completed all tasks.
 *	    }
 *	    else {
 *		lastTask=true;
 *		itinerary.setTask(new TaskB());
 *		itinerary.startTrip();
 *	    }
 *	}
 *  }
 * </pre>
 * The above code defines an aglet which performs 2-phase computation: in the
 * first phase, the aglet travels among multiple destinations to perform a task 
 * defined by TaskA. Then, the second phase is started, in which the aglet visits 
 * again all these destinations to perform a task of TaskB. 
 * <p>
 * In the above code, the invocation of <tt>itinerary.startTrip()</tt> causes the 
 * owner aglet to be dispatched sequentially among the destinations.
 * In every destination, the <tt>execute()</tt> of the corresponding <tt>Task</tt> 
 * object (assigned to the <tt>SlaveItinerary</tt> object via <tt>setTask()</tt>) 
 * is automatically invoked.
 * 
 * @version     1.20    $Date: 98/04/14 5:17p $
 * @author      Yariv Aridor
 */

public class SlaveItinerary extends SeqItinerary  {

    /**
     * Add a new address to the itinerary of the owner aglet
     */
    public void addPlan (String address) {
	addTask(address,task);
    }

    /**
     * Return the current task to be preformed by the owner aglet
     */
    public Task getTask () {
	return task;
    }

    /**
     * Set the task to be preformed by the owner aglet
     */
    public void setTask (Task task) {
	this.task=task;
	Enumeration e=addresses();
	clear();
	for (; e.hasMoreElements();) {
	    String address=(String)e.nextElement();
	    addPlan(address);
	}
    }

    /**
     * Constructor.
     * @param aglet the owner aglet
     * @param task the task to preform
     * @param address an address where the task should be preformed
     */
    public SlaveItinerary (Aglet aglet, String address, Task task) {
	super(aglet);
	this.task=task;
	addPlan(address);
    }

    /**
     * Constructor.
     * @param aglet the owner aglet
     * @param task the task to preform
     * @param addresses a vector of address where the task should be performed.
     */
    public SlaveItinerary(Aglet aglet, Vector addresses, Task task) {
	super(aglet);
	this.task=task;
	for (Enumeration e=addresses.elements(); e.hasMoreElements();) {
	    addPlan( ((URL)e.nextElement()).toString() );
	}
    }

    private Task task = null;
}
